#version 330 core 

in vec2 TexCoord;

in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;
in vec3 aNormal;

out vec4 color;

uniform sampler2D _textureSampler;
uniform samplerCube depthMap;

uniform bool hasSpec;
uniform sampler2D _specularSampler;

uniform mat4 model;
uniform mat4 view;


//light properties
uniform vec3 LightPosition_worldspace;
uniform vec4 LightColor;
uniform float LightPower;

uniform vec3 viewPos;
uniform float far_plane;

// array of offset direction for sampling
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
);

float ShadowCalculation(vec3 fragPos)
{
   // get vector between fragment position and light position
    vec3 fragToLight = fragPos - LightPosition_worldspace;
    
    // now get current linear depth as the length between the fragment and light position
    float currentDepth = length(fragToLight);
    
    // PCF    
	float shadow = 0.0;
    float bias = 0.10;
    int samples = 20;
    float viewDistance = length(viewPos - fragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
    for(int i = 0; i < samples; ++i)
    {
        float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
        closestDepth *= far_plane;   // undo mapping [0;1]
        if(currentDepth - bias > closestDepth)
            shadow += 1.0;
    }
    shadow /= float(samples);
  
    return shadow;
}

void main()
{
	// Material properties
	vec4 MaterialDiffuseColor	= texture(_textureSampler, TexCoord).rgba;
	vec4 MaterialAmbientColor	= vec4(0.5, 0.5, 0.5, 1.) * MaterialDiffuseColor;
	vec4 MaterialSpecularColor	= vec4(0.3,0.3,0.3,1.);

	// Distance to the light
	float distance = length( LightPosition_worldspace - Position_worldspace );

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( Normal_cameraspace );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	// Cosine of the angle between the normal and the light direction, 
	// clamped above 0
	//  - light is at the vertical of the triangle -> 1
	//  - light is perpendicular to the triangle -> 0
	//  - light is behind the triangle -> 0
	float cosTheta = clamp( dot( n,l ), 0,1 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize(EyeDirection_cameraspace);
	// Direction in which the triangle reflects the light
	vec3 R = reflect(-l,n);
	// Cosine of the angle between the Eye vector and the Reflect vector,
	// clamped to 0
	//  - Looking into the reflection -> 1
	//  - Looking elsewhere -> < 1
	float cosAlpha = clamp( dot( E,R ), 0,1 );
	
	// Ambient : simulates indirect lighting
	color = MaterialAmbientColor;
	
	// calculate shadow
    float shadow =  ShadowCalculation(Position_worldspace);  
	// Diffuse : "color" of the object
	color = color + MaterialDiffuseColor * (1.0-shadow) *LightColor * LightPower * cosTheta;// (distance*distance);
	
	// Specular : reflective highlight, like a mirror

	color = color + MaterialSpecularColor *  (1.0-shadow) * LightColor * LightPower * pow(cosAlpha,5); // (distance*distance);
}
